import os

from pacsman import DcmtkDicomClient, PynetDicomClient
from pacsman import dicom_file_iterator

TEST_DATA_DIRECTORY = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'test_dicom_data')
LOCAL_PACS_URL = os.environ.get('LOCAL_PACS_URL', 'localhost')


def main():
    print(f'uploading test data from {TEST_DATA_DIRECTORY}')

    # "TEST" must be registered AE in the Horos listener
    client = DcmtkDicomClient(client_ae='TEST', pacs_url=LOCAL_PACS_URL, pacs_port=11112,
                              dicom_dir='.', remote_ae='HOROS-LOCAL')
    client.send_datasets(dicom_file_iterator(TEST_DATA_DIRECTORY))

    client = PynetDicomClient(client_ae='TEST', pacs_url=LOCAL_PACS_URL, pacs_port=11112,
                              dicom_dir='.', remote_ae='HOROS-LOCAL')
    client.send_datasets(dicom_file_iterator(TEST_DATA_DIRECTORY))


if __name__ == '__main__':
    main()



import numpy as np
from pydicom import Dataset

from .utils import _scale_and_window_pixel_array_to_uint8, _pad_pixel_array_to_square, \
    copy_dicom_attributes, dicom_filename


def test_scale_pixel_array_to_png():
    arr = np.array([[0.5, 0.5], [1., 1.]], dtype=float)
    scaled = _scale_and_window_pixel_array_to_uint8(arr, 0.5, 1, 1, 0)
    assert np.array_equal(scaled, np.array([[0, 0], [255, 255]]))


def test_pad_png_pixel_array_already_square():
    arr = np.array([[1, 1, 1], [2, 2, 2], [3, 3, 3]])
    padded = _pad_pixel_array_to_square(arr)
    assert np.array_equal(arr, padded)


def test_pad_png_pixel_array_pad_right():
    arr = np.array([[1, 1], [2, 2], [3, 3]])
    padded = _pad_pixel_array_to_square(arr)
    expected = np.array([[1, 1, 255], [2, 2, 255], [3, 3, 255]])
    assert np.array_equal(padded, expected)


def test_pad_png_pixel_array_pad_down():
    arr = np.array([[1, 1, 1], [2, 2, 2]])
    padded = _pad_pixel_array_to_square(arr)
    expected = np.array([[1, 1, 1], [2, 2, 2], [255, 255, 255]])
    assert np.array_equal(padded, expected)


def test_copy_dicom_attributes():
    source_dataset = Dataset()
    destination_dataset = Dataset()
    destination_dataset.PatientName = 'Fred'
    additional_tags = ['PatientName']
    copy_dicom_attributes(destination_dataset, source_dataset, additional_tags)
    assert destination_dataset.PatientName == 'Fred'


def test_datasets_native_getattr_works():
    '''
    If this test fails, then that means you are using an older version of
    pydicom that has a bug causing `getattr` to ignore default values.
    '''
    assert getattr(Dataset(), 'PatientName', None) is None


def test_dicom_filename():
    dataset = Dataset()
    dataset.SOPInstanceUID = 'abracadabra'
    assert 'abracadabra.dcm' == dicom_filename(dataset)



import os
from typing import Iterable

import numpy as np
import png
import scipy.ndimage
from pydicom import Dataset, dcmread
from pydicom.multival import MultiValue
from pydicom.errors import InvalidDicomError


def process_and_write_png_from_file(thumbnail_dcm_path):
    '''
    :param thumbnail_dcm_path: DICOM instance file path. Must be unique per instance.
        The file is deleted to make the PNG.
    :return path to png (or None on failure). Uses the same name as the input file.
    '''
    if not os.path.exists(thumbnail_dcm_path):
        return None
    png_path = None
    try:
        thumbnail_ds = dcmread(thumbnail_dcm_path)
        png_path = os.path.splitext(thumbnail_dcm_path)[0] + '.png'
        process_and_write_png(thumbnail_ds, png_path)
    finally:
        os.remove(thumbnail_dcm_path)
    return png_path


def process_and_write_png(thumbnail_ds, png_path):
    '''
    :param thumbnail_ds: DICOM instance dataset with pixel array
    :param png_data: Output path for the thumbnail PNG

    Pads the instance pixel array with white to make it square, then scale to 100x100,
    and write out to png_path.
    '''
    thumbnail_slice = thumbnail_ds.pixel_array.astype(float)

    center_attr = dataset_attribute_fetcher(thumbnail_ds, 'WindowCenter')
    width_attr = dataset_attribute_fetcher(thumbnail_ds, 'WindowWidth')
    if center_attr and width_attr:
        center = center_attr[0] if isinstance(center_attr, MultiValue) else center_attr
        width = width_attr[0] if isinstance(width_attr, MultiValue) else width_attr
    else:
        # this is a CT soft tissue windowing in HU
        center = 40
        width = 400
    floor, roof = center - width / 2, center + width / 2,

    # RescaleSlope and RescaleIntercept have a defined VM of 1, but some PACS may not respect it
    slope_attr = getattr(thumbnail_ds, 'RescaleSlope', 1)
    slope = slope_attr[0] if isinstance(slope_attr, MultiValue) else slope_attr
    slope = float(slope)

    intercept_attr = getattr(thumbnail_ds, 'RescaleIntercept', 0)
    intercept = intercept_attr[0] if isinstance(intercept_attr, MultiValue) else intercept_attr
    intercept = float(intercept)

    png_scaled = _scale_and_window_pixel_array_to_uint8(thumbnail_slice, floor, roof,
                                                        slope, intercept)
    padded = _pad_pixel_array_to_square(png_scaled)

    # zoom to 100x100
    zoom_factor = 100 / padded.shape[0]
    png_array = scipy.ndimage.zoom(padded, zoom_factor, order=1)

    with open(png_path, 'wb') as f:
        writer = png.Writer(len(png_array[0]), len(png_array), greyscale=True)
        writer.write(f, png_array)


def _scale_and_window_pixel_array_to_uint8(arr, floor, roof, slope, intercept):
    '''
    Scales input float pixel array to 8 bit int for PNG writing.
    :param arr: stored value ndarray with type float
    :param floor: floor of window, values below are 0 / black
    :param roof: roof of window, values above are 255 / white
    :param slope: rescale slope to convert stored values to output units (e.g. HU for CT)
    :param intercept: rescale intercept to convert stored values to rescale type units
    :return: uint8 ndarray with same dimensions as input scaled between 0 and 255
    '''
    arr = arr * slope + intercept
    # png needs int values between 0 and 255
    result = np.zeros(arr.shape)
    result[arr >= roof] = 255
    inside_window = np.logical_and(arr > floor, arr < roof)
    result[inside_window] = (arr[inside_window] - floor) / (roof - floor) * 255
    return np.uint8(result)


def _pad_pixel_array_to_square(arr, pad_value=255):
    '''
    Pads the instance pixel array with value to make it square.
    Default is 255 (white for PNG)
    :param arr: Input scaled int ndarray
    :return: Square array padded with `pad_value`
    '''
    (a, b) = arr.shape
    if a > b:
        padding = ((0, 0), (0, a - b))
    else:
        padding = ((0, b - a), (0, 0))
    return np.pad(arr, padding, mode='constant', constant_values=pad_value)


def set_undefined_tags_to_blank(dataset, additional_tags):
    for tag in additional_tags or []:
        if not hasattr(dataset, tag) or getattr(dataset, tag) is None:
            setattr(dataset, tag, '')


def copy_dicom_attributes(destination, source, tags, missing='skip'):
    for tag in tags or []:
        if hasattr(source, tag):
            value = getattr(source, tag)
            setattr(destination, tag, value)
        elif missing == 'empty':
            setattr(destination, tag, '')
        elif missing != 'skip':
            raise ValueError(f'missing must be "skip" or "empty", not "{missing}"')


def dataset_attribute_fetcher(dataset, data_attribute):
    try:
        return getattr(dataset, data_attribute)
    except AttributeError:
        # Dataset has a bug where it ignores the default=None when getattr is called.
        return None


def dicom_file_iterator(folder: str) -> Iterable[Dataset]:
    for root, dirs, files in os.walk(folder):
        for file in files:
            dicom_file = os.path.join(root, file)
            try:
                dataset = dcmread(dicom_file)
                yield dataset
            except InvalidDicomError:
                pass


def dicom_filename(dataset: Dataset) -> str:
    return f'{dataset.SOPInstanceUID}.dcm'


def getattr_required(dataset, name):
    '''
    Helper function that should be used when accessing a required DICOM
    attribute, which should raise our standard exception upon a failure.
    '''
    try:
        return getattr(dataset, name)
    except AttributeError:
        raise InvalidDicomError(f"Missing required DICOM attribute {name}")



"""
A setuptools based setup module.
"""

from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

description = 'pacsman: Picture Archiving and Communication System Manager And Numpifier'

setup(
    name='pacsman',
    version='0.1.0',
    description=description,
    long_description=description,
    url='https://github.com/innolitics/pacsman',
    author='Innolitics, LLC',
    author_email='info@innolitics.com',
    license='MIT',
    classifiers=[
        'Development Status :: 1 - Planning',

        'Intended Audience :: Developers',
        'Intended Audience :: Healthcare Industry',
        'Topic :: Scientific/Engineering :: Medical Science Apps.',
        'Topic :: Communications',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],

    keywords='scientific image',

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    install_requires=['pydicom>=1.3', 'pynetdicom>=2', 'numpy', 'pypng', 'scipy'],

    dependency_links=['git+https://github.com/pydicom/pynetdicom3.git#egg=pynetdicom3'],

    extras_require={},

    package_data={},
    data_files=[],
)



import pytest
import os

from .filesystem_dev_client import FilesystemDicomClient


@pytest.fixture(scope="module")
def filesystem_client():
    file_dir = os.path.dirname(os.path.abspath(__file__))
    dicom_source_dir = os.path.join(file_dir, 'test_dicom_data')
    return FilesystemDicomClient(dicom_dir='.', dicom_source_dir=dicom_source_dir,
                                 client_ae="asdf")


def test_no_study_date_filter(filesystem_client):
    studies = filesystem_client.studies_for_patient('N-Lymphoma')
    assert len(studies) == 5


def test_study_date_filter_none(filesystem_client):
    date_filter = '20190101-20200101'
    studies = filesystem_client.studies_for_patient('N-Lymphoma', date_filter)
    assert len(studies) == 0


def test_study_date_filter_one(filesystem_client):
    date_filter = '20000101-20000102'
    studies = filesystem_client.studies_for_patient('N-Lymphoma', date_filter)
    assert len(studies) == 1


def test_study_date_filter_one_include_end(filesystem_client):
    date_filter = '19991231-20000101'
    studies = filesystem_client.studies_for_patient('N-Lymphoma', date_filter)
    assert len(studies) == 1


def test_study_date_filter_some(filesystem_client):
    date_filter = '20000101-20000305'
    studies = filesystem_client.studies_for_patient('N-Lymphoma', date_filter)
    assert len(studies) == 3


def test_study_date_filter_all(filesystem_client):
    date_filter = '20000101-20010101'
    studies = filesystem_client.studies_for_patient('N-Lymphoma', date_filter)
    assert len(studies) == 5
